@@ -3,26 +3,22 @@ from typing import Optional, Union
 from sanic.helpers import STATUS_CODES
 
 
-_sanic_exceptions = {}
-
-
-def add_status_code(code, quiet=None):
-    """
-    Decorator used for adding exceptions to :class:`SanicException`.
-    """
-
-    def class_decorator(cls):
-        cls.status_code = code
-        if quiet or quiet is None and code != 500:
-            cls.quiet = True
-        _sanic_exceptions[code] = cls
-        return cls
-
-    return class_decorator
-
-
 class SanicException(Exception):
-    def __init__(self, message, status_code=None, quiet=None):
+    message: str = ""
+
+    def __init__(
+        self,
+        message: Optional[Union[str, bytes]] = None,
+        status_code: Optional[int] = None,
+        quiet: Optional[bool] = None,
+    ) -> None:
+        if message is None:
+            if self.message:
+                message = self.message
+            elif status_code is not None:
+                msg: bytes = STATUS_CODES.get(status_code, b"")
+                message = msg.decode("utf8")
+
         super().__init__(message)
 
         if status_code is not None:
@@ -33,45 +29,45 @@ class SanicException(Exception):
             self.quiet = True
 
 
-@add_status_code(404)
 class NotFound(SanicException):
     """
     **Status**: 404 Not Found
     """
 
-    pass
+    status_code = 404
+    quiet = True
 
 
-@add_status_code(400)
 class InvalidUsage(SanicException):
     """
     **Status**: 400 Bad Request
     """
 
-    pass
+    status_code = 400
+    quiet = True
 
 
-@add_status_code(405)
 class MethodNotSupported(SanicException):
     """
     **Status**: 405 Method Not Allowed
     """
 
+    status_code = 405
+    quiet = True
+
     def __init__(self, message, method, allowed_methods):
         super().__init__(message)
         self.headers = {"Allow": ", ".join(allowed_methods)}
 
 
-@add_status_code(500)
 class ServerError(SanicException):
     """
     **Status**: 500 Internal Server Error
     """
 
-    pass
+    status_code = 500
 
 
-@add_status_code(503)
 class ServiceUnavailable(SanicException):
     """
     **Status**: 503 Service Unavailable
@@ -80,7 +76,8 @@ class ServiceUnavailable(SanicException):
     down for maintenance). Generally, this is a temporary state.
     """
 
-    pass
+    status_code = 503
+    quiet = True
 
 
 class URLBuildError(ServerError):
@@ -88,7 +85,7 @@ class URLBuildError(ServerError):
     **Status**: 500 Internal Server Error
     """
 
-    pass
+    status_code = 500
 
 
 class FileNotFound(NotFound):
@@ -102,7 +99,6 @@ class FileNotFound(NotFound):
         self.relative_url = relative_url
 
 
-@add_status_code(408)
 class RequestTimeout(SanicException):
     """The Web server (running the Web site) thinks that there has been too
     long an interval of time between 1) the establishment of an IP
@@ -112,16 +108,17 @@ class RequestTimeout(SanicException):
     server has 'timed out' on that particular socket connection.
     """
 
-    pass
+    status_code = 408
+    quiet = True
 
 
-@add_status_code(413)
 class PayloadTooLarge(SanicException):
     """
     **Status**: 413 Payload Too Large
     """
 
-    pass
+    status_code = 413
+    quiet = True
 
 
 class HeaderNotFound(InvalidUsage):
@@ -129,36 +126,42 @@ class HeaderNotFound(InvalidUsage):
     **Status**: 400 Bad Request
     """
 
-    pass
+
+class InvalidHeader(InvalidUsage):
+    """
+    **Status**: 400 Bad Request
+    """
 
 
-@add_status_code(416)
 class ContentRangeError(SanicException):
     """
     **Status**: 416 Range Not Satisfiable
     """
 
+    status_code = 416
+    quiet = True
+
     def __init__(self, message, content_range):
         super().__init__(message)
         self.headers = {"Content-Range": f"bytes */{content_range.total}"}
 
 
-@add_status_code(417)
 class HeaderExpectationFailed(SanicException):
     """
     **Status**: 417 Expectation Failed
     """
 
-    pass
+    status_code = 417
+    quiet = True
 
 
-@add_status_code(403)
 class Forbidden(SanicException):
     """
     **Status**: 403 Forbidden
     """
 
-    pass
+    status_code = 403
+    quiet = True
 
 
 class InvalidRangeType(ContentRangeError):
@@ -166,7 +169,8 @@ class InvalidRangeType(ContentRangeError):
     **Status**: 416 Range Not Satisfiable
     """
 
-    pass
+    status_code = 416
+    quiet = True
 
 
 class PyFileError(Exception):
@@ -174,7 +178,6 @@ class PyFileError(Exception):
         super().__init__("could not execute config file %s", file)
 
 
-@add_status_code(401)
 class Unauthorized(SanicException):
     """
     **Status**: 401 Unauthorized
@@ -210,6 +213,9 @@ class Unauthorized(SanicException):
                            realm="Restricted Area")
     """
 
+    status_code = 401
+    quiet = True
+
     def __init__(self, message, status_code=None, scheme=None, **kwargs):
         super().__init__(message, status_code)
 
@@ -231,6 +237,11 @@ class InvalidSignal(SanicException):
     pass
 
 
+class WebsocketClosed(SanicException):
+    quiet = True
+    message = "Client has closed the websocket connection"
+
+
 def abort(status_code: int, message: Optional[Union[str, bytes]] = None):
     """
     Raise an exception based on SanicException. Returns the HTTP response
@@ -241,9 +252,13 @@ def abort(status_code: int, message: Optional[Union[str, bytes]] = None):
     :param status_code: The HTTP status code to return.
     :param message: The HTTP response body. Defaults to the messages in
     """
-    if message is None:
-        msg: bytes = STATUS_CODES[status_code]
-        # These are stored as bytes in the STATUS_CODES dict
-        message = msg.decode("utf8")
-    sanic_exception = _sanic_exceptions.get(status_code, SanicException)
-    raise sanic_exception(message=message, status_code=status_code)
+    import warnings
+
+    warnings.warn(
+        "sanic.exceptions.abort has been marked as deprecated, and will be "
+        "removed in release 21.12.\n To migrate your code, simply replace "
+        "abort(status_code, msg) with raise SanicException(msg, status_code), "
+        "or even better, raise an appropriate SanicException subclass."
+    )
+
+    raise SanicException(message=message, status_code=status_code)
